import networkx as nx
import matplotlib.pyplot as plt
from sklearn.feature_extraction.text import TfidfVectorizer
from collections import Counter
import re
import random

# Function to preprocess text
def preprocess_text(text):
    text = re.sub(r'\W', ' ', text)  # Remove non-alphanumeric characters
    text = re.sub(r'\s+', ' ', text)  # Replace multiple spaces with a single space
    return text.lower()

# Function to extract keywords from a text
def extract_keywords(text, top_n=50):
    vectorizer = TfidfVectorizer(stop_words='english', preprocessor=preprocess_text)
    tfidf_matrix = vectorizer.fit_transform([text])
    feature_names = vectorizer.get_feature_names_out()
    feature_weights = tfidf_matrix.sum(axis=0).A1
    keywords = [feature_names[i] for i in feature_weights.argsort()[-top_n:][::-1]]
    return keywords

# Function to draw network graph
def draw_network_graph(keywords):
    G = nx.Graph()
    G.add_nodes_from(keywords)

    # Create arbitrary connections
    for node in keywords:
        num_connections = random.randint(2, 10)
        connections = random.sample(keywords, min(num_connections, len(keywords)))
        for connection in connections:
            if node != connection:
                G.add_edge(node, connection)

    # Draw the graph with unique colors for each node
    plt.figure(figsize=(10, 8))
    colors = range(len(G.nodes))
    nx.draw(G, with_labels=True, font_size=8, node_size=700, cmap=plt.cm.plasma,
            node_color=colors, font_color='black', font_weight='bold', edge_color='gray', alpha=0.7)

    plt.title("Keyword Network Between 50 Related Keywords")
    plt.show()

# Read text from file
file_path = '/content/titles.txt'  # Replace with the actual file path
with open(file_path, 'r', encoding='utf-8') as file:
    text = file.read()

# Extract keywords
top_keywords = extract_keywords(text, top_n=50)

# Draw network graph
draw_network_graph(top_keywords)
